"""Special handler for modifier."""

from __future__ import annotations

from typing import TYPE_CHECKING

from ..strategy import VERSION_STRATEGIES_DICT, AwesomeVersionStrategy
from ..utils.regex import (
    RE_MODIFIER,
    extract_digits,
    match_compound_modifier,
    match_modifier_compound_part,
)

SEMVER_MODIFIER_MAP = {"dev": 0, "alpha": 1, "beta": 2, "rc": 3}

if TYPE_CHECKING:
    from awesomeversion import AwesomeVersion


def _extract_modifier_info(version_string: str) -> tuple[str | None, str | None]:
    """Extract modifier number and full modifier string from version."""
    semver_pattern = VERSION_STRATEGIES_DICT[AwesomeVersionStrategy.SEMVER].pattern
    semver_match = semver_pattern.match(version_string)

    if not semver_match or len(semver_match.groups()) < 4:
        return None, None

    full_modifier = semver_match.group(4)
    modifier_match = RE_MODIFIER.match(full_modifier)

    if modifier_match and len(modifier_match.groups()) >= 4:
        modifier_num = modifier_match.group(4)
    else:
        compound_match = match_compound_modifier(full_modifier)
        modifier_num = compound_match.group(2) if compound_match else None

    return modifier_num, full_modifier


def _compare_modifier_types(type_a: str, type_b: str) -> bool:
    """Compare two modifier types using SEMVER_MODIFIER_MAP or alphabetically."""
    priority_a = SEMVER_MODIFIER_MAP.get(type_a)
    priority_b = SEMVER_MODIFIER_MAP.get(type_b)

    if priority_a is not None and priority_b is not None:
        return priority_a > priority_b

    return type_a > type_b


def _compare_compound_parts(part_a: str, part_b: str) -> bool | None:
    """Compare two compound modifier parts. Returns None if equal."""
    mod_match_a = match_modifier_compound_part(part_a)
    mod_match_b = match_modifier_compound_part(part_b)

    if mod_match_a and mod_match_b:
        type_a, num_a = mod_match_a.groups()
        type_b, num_b = mod_match_b.groups()

        if type_a != type_b:
            return _compare_modifier_types(type_a, type_b)

        if num_a and num_b:
            num_val_a, num_val_b = int(num_a), int(num_b)
            if num_val_a != num_val_b:
                return num_val_a > num_val_b

        return None

    nums_a = extract_digits(part_a)
    nums_b = extract_digits(part_b)

    if nums_a and nums_b:
        num_val_a, num_val_b = int(nums_a[0]), int(nums_b[0])
        if num_val_a != num_val_b:
            return num_val_a > num_val_b

    return None


def _compare_compound_modifiers(full_mod_a: str, full_mod_b: str) -> bool | None:
    """Compare compound modifiers after the primary part."""
    parts_a = full_mod_a.split("-")[1:]
    parts_b = full_mod_b.split("-")[1:]

    for part_a, part_b in zip(parts_a, parts_b):
        result = _compare_compound_parts(part_a, part_b)
        if result is not None:
            return result

    return len(parts_a) > len(parts_b) if len(parts_a) != len(parts_b) else None


def compare_handler_semver_modifier(
    version_a: AwesomeVersion,
    version_b: AwesomeVersion,
) -> bool | None:
    """Compare handler sections."""
    if (
        AwesomeVersionStrategy.SEMVER not in (version_a.strategy, version_b.strategy)
        or version_a.modifier_type is None
        or version_b.modifier_type is None
    ):
        return None

    if version_a.modifier_type != version_b.modifier_type:
        mod_a = SEMVER_MODIFIER_MAP.get(version_a.modifier_type)
        mod_b = SEMVER_MODIFIER_MAP.get(version_b.modifier_type)
        if mod_a is not None and mod_b is not None:
            return mod_a > mod_b

    mod_num_a, full_mod_a = _extract_modifier_info(version_a.string)
    mod_num_b, full_mod_b = _extract_modifier_info(version_b.string)

    result = None
    if mod_num_a and mod_num_b:
        primary_diff = int(mod_num_a) - int(mod_num_b)
        if primary_diff != 0:
            result = primary_diff > 0
    elif mod_num_a and not mod_num_b:
        result = False
    elif not mod_num_a and mod_num_b:
        result = True

    if result is None and full_mod_a and full_mod_b:
        compound_result = _compare_compound_modifiers(full_mod_a, full_mod_b)
        if compound_result is not None:
            result = compound_result

    return result if result is not None else False



"""Regex utils for AwesomeVersion."""

from __future__ import annotations

from functools import lru_cache
import re
from typing import Pattern

RE_IS_SINGLE_DIGIT = re.compile(r"^\d$")
RE_DIGIT = re.compile(r"[a-z]*(\d+)[a-z]*")
RE_MODIFIER = re.compile(r"^((?:\d+\-|\d|))(([a-z]+)\.?(\d*))$")
RE_COMPOUND_MODIFIER = re.compile(r"^([a-z]+)(\d*)-.*$")
RE_MODIFIER_COMPOUND_PART = re.compile(r"^([a-z]+)(\d*)$")


RE_CALVER = r"(\d{2}|\d{4})\.\d{1,2}?(\.?\d{1,2}?\.?)?(\.\d)?(\d*(\w+\d+)?)"

RE_SEMVER = (
    r"(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)"
    r"(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?"
)


RE_PEP440 = (
    r"([1-9][0-9]*!)?(0|[1-9][0-9]*)(\.(0|[1-9][0-9]*))*"
    r"([-_\.]?(?:alpha|beta|c|pre|preview|a|b|rc)(0|[1-9][0-9]*))?"
    r"([-_\.]?(?:post|r|rev)(0|[1-9][0-9]*))?"
    r"([-_\.]?(?:d|dev)(0|[1-9][0-9]*))?"
    r"(?:\+([a-z0-9]+(?:[-_\.][a-z0-9]+)*))?"
)

RE_BUILDVER = r"\d+"
RE_HEXVER = r"0x[A-Fa-f0-9]+"

RE_SPECIAL_CONTAINER = r"(?:latest|dev|stable|beta)"

RE_SIMPLE = r"[vV]?((\d+)(\.\d+)+)"


@lru_cache(maxsize=128)
def compile_regex(pattern: str) -> Pattern[str]:
    """Compile a regex with caching to avoid recompilation."""
    return re.compile(pattern)


@lru_cache(maxsize=32)
def generate_full_string_regex(string: str) -> Pattern[str]:
    """Generate a regex that matches the full string with caching."""
    return compile_regex(r"^" + string + r"$")


@lru_cache(maxsize=64)
def match_compound_modifier(modifier: str) -> re.Match[str] | None:
    """Match compound modifiers with caching for performance."""
    return RE_COMPOUND_MODIFIER.match(modifier)


@lru_cache(maxsize=128)
def match_modifier_compound_part(part: str) -> re.Match[str] | None:
    """Match modifier parts with caching for performance."""
    return RE_MODIFIER_COMPOUND_PART.match(part)


@lru_cache(maxsize=256)
def extract_digits(text: str) -> list[str]:
    """Extract all digits from text with caching for performance."""
    return re.findall(r"(\d+)", text)


_COMPILED_PATTERNS = {
    "BUILDVER_FULL": re.compile(r"^\d+$"),
    "HEXVER_FULL": re.compile(r"^0x[A-Fa-f0-9]+$"),
    "SPECIAL_CONTAINER_FULL": re.compile(r"^(?:latest|dev|stable|beta)$"),
    "IS_SINGLE_DIGIT": RE_IS_SINGLE_DIGIT,
    "DIGIT": RE_DIGIT,
    "MODIFIER": RE_MODIFIER,
}


def get_compiled_pattern(pattern_name: str) -> Pattern[str] | None:
    """Get a pre-compiled pattern for maximum performance."""
    return _COMPILED_PATTERNS.get(pattern_name)
